@@ -46,7 +46,9 @@ class Expr:
 
     operands: list
 
-    def __new__(cls, *args, **kwargs):
+    _determ_token: str | None
+
+    def __new__(cls, *args, _determ_token=None, **kwargs):
         operands = list(args)
         for parameter in cls._parameters[len(operands) :]:
             try:
@@ -55,6 +57,8 @@ class Expr:
                 operands.append(cls._defaults[parameter])
         assert not kwargs, kwargs
         inst = object.__new__(cls)
+
+        inst._determ_token = _determ_token
         inst.operands = [_unpack_collections(o) for o in operands]
         _name = inst._name
         if _name in Expr._instances:
@@ -127,10 +131,17 @@ class Expr:
     def __dask_tokenize__(self):
         return self._name
 
+    @staticmethod
+    def _reconstruct(*args):
+        typ, *operands, token = args
+        return typ(*operands, _determ_token=token)
+
     def __reduce__(self):
         if dask.config.get("dask-expr-no-serialize", False):
             raise RuntimeError(f"Serializing a {type(self)} object")
-        return type(self), tuple(self.operands)
+        return Expr._reconstruct, tuple(
+            [type(self)] + self.operands + [self.deterministic_token]
+        )
 
     def _depth(self, cache=None):
         """Depth of the expression tree
@@ -153,6 +164,19 @@ class Expr:
                     cache[expr._name] = result[-1]
             return max(result)
 
+    def __setattr__(self, name: str, value: Any) -> None:
+        if name in ["operands", "_determ_token"]:
+            object.__setattr__(self, name, value)
+            return
+        try:
+            params = object.__getattribute__(type(self), "_parameters")
+            operands = object.__getattribute__(self, "operands")
+            operands[params.index(name)] = value
+        except ValueError:
+            raise AttributeError(
+                f"{type(self).__name__} object has no attribute {name}"
+            )
+
     def operand(self, key):
         # Access an operand unambiguously
         # (e.g. if the key is reserved by a method/property)
@@ -220,7 +244,7 @@ class Expr:
 
         return {
             (self._name, i): self._task((self._name, i), i)
-            for i in range(self.npartitions)  # type: ignore
+            for i in range(self.npartitions)
         }
 
     def rewrite(self, kind: str, rewritten):
@@ -460,9 +484,15 @@ class Expr:
     def _funcname(self) -> str:
         return funcname(type(self)).lower()
 
+    @property
+    def deterministic_token(self):
+        if not self._determ_token:
+            self._determ_token = _tokenize_deterministic(*self.operands)
+        return self._determ_token
+
     @functools.cached_property
     def _name(self) -> str:
-        return self._funcname + "-" + _tokenize_deterministic(*self.operands)
+        return self._funcname + "-" + self.deterministic_token
 
     @property
     def _meta(self):
@@ -733,6 +763,32 @@ class Expr:
         ), "`operation` must be`Expr` subclass)"
         return (expr for expr in self.walk() if isinstance(expr, operation))
 
+    def __getattr__(self, key):
+        try:
+            return object.__getattribute__(self, key)
+        except AttributeError as err:
+            if key.startswith("_meta"):
+                # Avoid a recursive loop if/when `self._meta*`
+                # produces an `AttributeError`
+                raise RuntimeError(
+                    f"Failed to generate metadata for {self}. "
+                    "This operation may not be supported by the current backend."
+                )
+
+            # Allow operands to be accessed as attributes
+            # as long as the keys are not already reserved
+            # by existing methods/properties
+            _parameters = type(self)._parameters
+            if key in _parameters:
+                idx = _parameters.index(key)
+                return self.operands[idx]
+
+            raise AttributeError(
+                f"{err}\n\n"
+                "This often means that you are attempting to use an unsupported "
+                f"API function.."
+            )
+
 
 def collect_dependents(expr) -> defaultdict:
     dependents = defaultdict(list)
