@@ -8,15 +8,25 @@ from typing import Any
 import numpy as np
 
 from dask import config
+from dask._task_spec import (
+    Alias,
+    DataNode,
+    GraphNode,
+    Task,
+    _execute_subgraph,
+    convert_legacy_graph,
+    fuse_linear_task_spec,
+)
 from dask.array.chunk import getitem
 from dask.array.core import getter, getter_inline, getter_nofancy
 from dask.blockwise import fuse_roots, optimize_blockwise
-from dask.core import flatten, reverse_dict
+from dask.core import flatten
 from dask.highlevelgraph import HighLevelGraph
-from dask.optimization import SubgraphCallable, fuse, inline_functions
 from dask.utils import ensure_dict
 
 # All get* functions the optimizations know about
+
+
 GETTERS = (getter, getter_nofancy, getter_inline, getitem)
 # These get* functions aren't ever completely removed from the graph,
 # even if the index should be a no-op by numpy semantics. Some array-like's
@@ -24,20 +34,11 @@ GETTERS = (getter, getter_nofancy, getter_inline, getitem)
 GETNOREMOVE = (getter, getter_nofancy)
 
 
-def optimize(
-    dsk,
-    keys,
-    fuse_keys=None,
-    fast_functions=None,
-    inline_functions_fast_functions=(getter_inline,),
-    rename_fused_keys=True,
-    **kwargs,
-):
+def optimize(dsk, keys, **kwargs):
     """Optimize dask for array computation
 
     1.  Cull tasks not necessary to evaluate keys
-    2.  Remove full slicing, e.g. x[:]
-    3.  Inline fast functions like getitem and np.transpose
+    2.  Perform linear fusion
     """
     if not isinstance(keys, (list, set)):
         keys = [keys]
@@ -55,186 +56,133 @@ def optimize(
     if config.get("optimization.fuse.active") is False:
         return dsk
 
-    dependencies = dsk.get_all_dependencies()
     dsk = ensure_dict(dsk)
 
-    # Low level task optimizations
-    if fast_functions is not None:
-        inline_functions_fast_functions = fast_functions
+    dsk = convert_legacy_graph(dsk)
+    dsk = fuse_linear_task_spec(dsk, keys=keys)
+    dsk = _optimize_slices(dsk)
 
-    hold = hold_keys(dsk, dependencies)
-
-    dsk, dependencies = fuse(
-        dsk,
-        hold + keys + (fuse_keys or []),
-        dependencies,
-        rename_keys=rename_fused_keys,
-    )
-    if inline_functions_fast_functions:
-        dsk = inline_functions(
-            dsk,
-            keys,
-            dependencies=dependencies,
-            fast_functions=inline_functions_fast_functions,
-        )
-
-    return optimize_slices(dsk)
-
-
-def hold_keys(dsk, dependencies):
-    """Find keys to avoid fusion
-
-    We don't want to fuse data present in the graph because it is easier to
-    serialize as a raw value.
-
-    We don't want to fuse chains after getitem/GETTERS because we want to
-    move around only small pieces of data, rather than the underlying arrays.
-    """
-    dependents = reverse_dict(dependencies)
-    data = {k for k, v in dsk.items() if type(v) not in (tuple, str)}
-
-    hold_keys = list(data)
-    for dat in data:
-        deps = dependents[dat]
-        for dep in deps:
-            task = dsk[dep]
-            # If the task is a get* function, we walk up the chain, and stop
-            # when there's either more than one dependent, or the dependent is
-            # no longer a get* function or an alias. We then add the final
-            # key to the list of keys not to fuse.
-            if _is_getter_task(task):
-                try:
-                    while len(dependents[dep]) == 1:
-                        new_dep = next(iter(dependents[dep]))
-                        new_task = dsk[new_dep]
-                        # If the task is a get* or an alias, continue up the
-                        # linear chain
-                        if _is_getter_task(new_task) or new_task in dsk:
-                            dep = new_dep
-                        else:
-                            break
-                except (IndexError, TypeError):
-                    pass
-                hold_keys.append(dep)
-    return hold_keys
+    return dsk
 
 
 def _is_getter_task(
-    value,
+    value: GraphNode,
 ) -> tuple[Callable, Any, Any, bool, bool | None] | None:
     """Check if a value in a Dask graph looks like a getter.
-
     1. Is it a tuple with the first element a known getter.
-    2. Is it a SubgraphCallable with a single element in its
-       dsk which is a known getter.
-
     If a getter is found, it returns a tuple with (getter, array, index, asarray, lock).
     Otherwise it returns ``None``.
-
-    TODO: the second check is a hack to allow for slice fusion between tasks produced
-    from blockwise layers and slicing operations. Once slicing operations have
-    HighLevelGraph layers which can talk to Blockwise layers this check *should* be
-    removed, and we should not have to introspect SubgraphCallables.
     """
-    if type(value) is not tuple:
+    if not isinstance(value, Task):
         return None
-    first = value[0]
-    get: Callable | None = None
-    if first in GETTERS:
-        get = first
-    # We only accept SubgraphCallables with a single sub-task right now as it's
-    # not clear which task to inspect if there is more than one, or how to resolve
-    # conflicts if they occur.
-    elif isinstance(first, SubgraphCallable) and len(first.dsk) == 1:
-        v = next(iter(first.dsk.values()))
-        if type(v) is tuple and len(v) > 1 and v[0] in GETTERS:
-            get = v[0]
-    if get is None:  # Didn't find a getter
+    func = value.func
+    if func in GETTERS:
+        get = func
+    else:
         return None
 
-    length = len(value)
-    if length == 3:
+    length = len(value.args)
+    if length == 2:
         # getter defaults to asarray=True, getitem is semantically False
-        return get, value[1], value[2], get is not getitem, None
-    elif length == 5:
-        return get, *value[1:]  # type: ignore
+        return get, value.args[0], value.args[1], get is not getitem, None
+    elif length == 4:
+        return get, *list(value.args)
 
     return None
 
 
-def optimize_slices(dsk):
+def _optimize_slices(dsk):
     """Optimize slices
-
     1.  Fuse repeated slices, like x[5:][2:6] -> x[7:11]
-    2.  Remove full slices, like         x[:] -> x
 
-    See also:
-        fuse_slice_dict
+    This is generally not very important since we are fusing those tasks anyway. There
+    is one specific exception to how xarray implements opening netcdf files and subsequent
+    slices. Not merging them together can cause reading the whole netcdf file before
+    we drop the unnecessary data. Fusing slices avoids that pattern.
+
+    See https://github.com/pydata/xarray/issues/9926
     """
     fancy_ind_types = (list, np.ndarray)
     dsk = dsk.copy()
-    for k, v in dsk.items():
-        if a_task := _is_getter_task(v):
-            get, a, a_index, a_asarray, a_lock = a_task
+    for _, v in dsk.items():
+        if not (isinstance(v, Task) and v.func is _execute_subgraph):
+            continue
 
-            while b_task := _is_getter_task(a):
-                f2, b, b_index, b_asarray, b_lock = b_task
+        inner_graph: dict = v.args[0]  # type: ignore
 
-                if a_lock and a_lock is not b_lock:
-                    break
-                if (type(a_index) is tuple) != (type(b_index) is tuple):
-                    break
-                if type(a_index) is tuple:
-                    indices = b_index + a_index
-                    if len(a_index) != len(b_index) and any(i is None for i in indices):
+        seen = set()
+
+        for inner_key, inner_value in inner_graph.items():
+            if inner_key in seen:
+                continue
+            else:
+                seen.add(inner_key)
+
+            if a_task := _is_getter_task(inner_value):
+                temp_key = inner_key
+                get, a, a_index, a_asarray, a_lock = a_task
+                fused = False
+
+                while a.key in inner_graph and (
+                    b_task := _is_getter_task(inner_graph[a.key])
+                ):
+                    seen.add(a.key)
+                    f2, b, b_index, b_asarray, b_lock = b_task
+
+                    if isinstance(a_index, DataNode):
+                        a_index = a_index.value
+                    if isinstance(b_index, DataNode):
+                        b_index = b_index.value
+
+                    if a_lock and a_lock is not b_lock:
                         break
-                    if f2 is getter_nofancy and any(
-                        isinstance(i, fancy_ind_types) for i in indices
+                    if (type(a_index) is tuple) != (type(b_index) is tuple):
+                        break
+                    if type(a_index) is tuple:
+                        indices = b_index + a_index
+                        if len(a_index) != len(b_index) and any(
+                            i is None for i in indices
+                        ):
+                            break
+                        if f2 is getter_nofancy and not any(
+                            isinstance(i, fancy_ind_types) for i in indices
+                        ):
+                            break
+                    elif f2 is getter_nofancy and (
+                        type(a_index) in fancy_ind_types
+                        or type(b_index) in fancy_ind_types
                     ):
                         break
-                elif f2 is getter_nofancy and (
-                    type(a_index) in fancy_ind_types or type(b_index) in fancy_ind_types
-                ):
-                    break
-                try:
-                    c_index = fuse_slice(b_index, a_index)
-                    # rely on fact that nested gets never decrease in
-                    # strictness e.g. `(getter_nofancy, (getter, ...))` never
-                    # happens
-                    get = getter if f2 is getter_inline else f2
-                except NotImplementedError:
-                    break
-                a, a_index, a_lock = b, c_index, b_lock
-                a_asarray |= b_asarray
+                    try:
+                        c_index = fuse_slice(b_index, a_index)
+                        # rely on fact that nested gets never decrease in
+                        # strictness e.g. `(getter_nofancy, (getter, ...))` never
+                        # happens
+                        get = getter if f2 is getter_inline else f2
 
-            # Skip the get call if not from from_array and nothing to do
-            if get not in GETNOREMOVE and (
-                (
-                    type(a_index) is slice
-                    and not a_index.start
-                    and a_index.stop is None
-                    and a_index.step is None
-                )
-                or (
-                    type(a_index) is tuple
-                    and all(
-                        type(s) is slice
-                        and not s.start
-                        and s.stop is None
-                        and s.step is None
-                        for s in a_index
+                        inner_graph[temp_key] = Alias(temp_key, a.key)
+                        temp_key = a.key
+                        fused = True
+                    except NotImplementedError:
+                        break
+                    a, a_index, a_lock = b, c_index, b_lock
+                    a_asarray |= b_asarray
+
+                    if not isinstance(a, Task):
+                        break
+
+                if not fused:
+                    pass
+                elif get is getitem or (a_asarray and not a_lock):
+                    # default settings are fine, drop the extra parameters Since we
+                    # always fallback to inner `get` functions, `get is getitem`
+                    # can only occur if all gets are getitem, meaning all
+                    # parameters must be getitem defaults.
+                    inner_graph[temp_key] = Task(temp_key, get, a, a_index)
+                else:
+                    inner_graph[temp_key] = Task(
+                        temp_key, get, a, a_index, a_asarray, a_lock
                     )
-                )
-            ):
-                dsk[k] = a
-            elif get is getitem or (a_asarray and not a_lock):
-                # default settings are fine, drop the extra parameters Since we
-                # always fallback to inner `get` functions, `get is getitem`
-                # can only occur if all gets are getitem, meaning all
-                # parameters must be getitem defaults.
-                dsk[k] = (get, a, a_index)
-            else:
-                dsk[k] = (get, a, a_index, a_asarray, a_lock)
 
     return dsk
 
