@@ -107,11 +107,7 @@ class FusedIO(BlockwiseIO):
 
     @functools.cached_property
     def _name(self):
-        return (
-            self.operand("_expr")._funcname
-            + "-fused-"
-            + _tokenize_deterministic(*self.operands)
-        )
+        return self.operand("_expr")._funcname + "-fused-" + self.deterministic_token
 
     @functools.cached_property
     def _meta(self):
@@ -167,7 +163,7 @@ class FusedParquetIO(FusedIO):
         return (
             funcname(type(self.operand("_expr"))).lower()
             + "-fused-parq-"
-            + _tokenize_deterministic(*self.operands)
+            + self.deterministic_token
         )
 
     @staticmethod
@@ -245,13 +241,9 @@ class FromMap(PartitionsFiltered, BlockwiseIO):
     @functools.cached_property
     def _name(self):
         if self.label is None:
-            return (
-                funcname(self.func).lower()
-                + "-"
-                + _tokenize_deterministic(*self.operands)
-            )
+            return funcname(self.func).lower() + "-" + self.deterministic_token
         else:
-            return self.label + "-" + _tokenize_deterministic(*self.operands)
+            return self.label + "-" + self.deterministic_token
 
     @functools.cached_property
     def _meta(self):
@@ -406,6 +398,7 @@ class FromPandas(PartitionsFiltered, BlockwiseIO):
         "pyarrow_strings_enabled",
         "_partitions",
         "_series",
+        "_pd_length_stats",
     ]
     _defaults = {
         "npartitions": None,
@@ -415,8 +408,9 @@ class FromPandas(PartitionsFiltered, BlockwiseIO):
         "_series": False,
         "chunksize": None,
         "pyarrow_strings_enabled": True,
+        "_pd_length_stats": None,
     }
-    _pd_length_stats = None
+    _pd_length_stats: tuple | None
     _absorb_projections = True
 
     @functools.cached_property
@@ -445,6 +439,8 @@ class FromPandas(PartitionsFiltered, BlockwiseIO):
             try:
                 return list(self.frame.columns)
             except AttributeError:
+                if self.ndim == 1:
+                    return [self.name]
                 return []
         else:
             return _convert_to_list(columns_operand)
@@ -544,13 +540,19 @@ class FromPandasDivisions(FromPandas):
         "pyarrow_strings_enabled",
         "_partitions",
         "_series",
+        "_pd_length_stats",
     ]
-    _defaults = {"columns": None, "_partitions": None, "_series": False}
+    _defaults = {
+        "columns": None,
+        "_partitions": None,
+        "_series": False,
+        "_pd_length_stats": None,
+    }
     sort = True
 
     @functools.cached_property
     def _name(self):
-        return "from_pd_divs" + "-" + _tokenize_deterministic(*self.operands)
+        return "from_pd_divs" + "-" + self.deterministic_token
 
     @property
     def _divisions_and_locations(self):
