@@ -16,6 +16,7 @@ import numpy as np
 import pandas as pd
 import pyarrow as pa
 import pyarrow.compute as pc
+import pyarrow.dataset
 import pyarrow.dataset as pa_ds
 import pyarrow.fs as pa_fs
 import pyarrow.parquet as pq
@@ -45,7 +46,7 @@ from dask.dataframe.dask_expr._expr import (
     determine_column_projection,
 )
 from dask.dataframe.dask_expr._reductions import Len
-from dask.dataframe.dask_expr._util import _convert_to_list, _tokenize_deterministic
+from dask.dataframe.dask_expr._util import _convert_to_list
 from dask.dataframe.dask_expr.io import BlockwiseIO, PartitionsFiltered
 from dask.dataframe.dask_expr.io.io import FusedParquetIO
 from dask.dataframe.io.parquet.core import (
@@ -60,7 +61,7 @@ from dask.dataframe.io.parquet.core import (
 from dask.dataframe.io.parquet.utils import _split_user_options
 from dask.dataframe.io.utils import _is_local_fs
 from dask.delayed import delayed
-from dask.tokenize import normalize_token, tokenize
+from dask.tokenize import _tokenize_deterministic, normalize_token, tokenize
 from dask.typing import Key
 from dask.utils import apply, funcname, natural_sort_key, parse_bytes, typename
 
@@ -564,10 +565,9 @@ def to_parquet(
     # If write_index==True (default), reset the index and record the
     # name of the original index in `index_cols` (we will set the name
     # to the NONE_LABEL constant if it is originally `None`).
-    # `fastparquet` will use `index_cols` to specify the index column(s)
-    # in the metadata.  `pyarrow` will revert the `reset_index` call
+    # `pyarrow` will revert the `reset_index` call
     # below if `index_cols` is populated (because pyarrow will want to handle
-    # index preservation itself).  For both engines, the column index
+    # index preservation itself).  The column index
     # will be written to "pandas metadata" if write_index=True
     index_cols = []
     if write_index:
@@ -630,26 +630,33 @@ def to_parquet(
     else:
         ctx = contextlib.nullcontext()
 
-    with ctx:
-        out = new_collection(
-            ToParquet(
-                df,
-                path,
-                fs,
-                fmd,
-                engine,
-                i_offset,
-                partition_on,
-                write_metadata_file,
-                name_function,
-                toolz.merge(
-                    kwargs,
-                    {"compression": compression, "custom_metadata": custom_metadata},
-                    extra_write_kwargs,
-                ),
-                append,
-            )
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore", message="Dask annotations ", category=UserWarning
         )
+        with ctx:
+            out = new_collection(
+                ToParquet(
+                    df,
+                    path,
+                    fs,
+                    fmd,
+                    engine,
+                    i_offset,
+                    partition_on,
+                    write_metadata_file,
+                    name_function,
+                    toolz.merge(
+                        kwargs,
+                        {
+                            "compression": compression,
+                            "custom_metadata": custom_metadata,
+                        },
+                        extra_write_kwargs,
+                    ),
+                    append,
+                )
+            )
 
     if compute:
         out = out.compute(**compute_kwargs)
@@ -710,7 +717,6 @@ def _determine_type_mapper(
 
 
 class ReadParquet(PartitionsFiltered, BlockwiseIO):
-    _pq_length_stats = None
     _absorb_projections = True
     _filter_passthrough = False
 
@@ -771,15 +777,18 @@ class ReadParquet(PartitionsFiltered, BlockwiseIO):
     def _funcname(self):
         return "read_parquet"
 
-    @cached_property
-    def _name(self):
-        return (
-            self._funcname
-            + "-"
-            + _tokenize_deterministic(
+    @property
+    def deterministic_token(self):
+        if not self._determ_token:
+            # TODO: Is there an actual need to overwrite this?
+            self._determ_token = _tokenize_deterministic(
                 funcname(type(self)), self.checksum, *self.operands[:-1]
             )
-        )
+        return self._determ_token
+
+    @cached_property
+    def _name(self):
+        return self._funcname + "-" + self.deterministic_token
 
     @property
     def checksum(self):
@@ -1031,14 +1040,15 @@ class ReadParquetPyarrowFS(ReadParquet):
             if all_files[-1].base_name.endswith("_metadata"):
                 metadata_file = all_files.pop()
                 checksum = tokenize(metadata_file)
-                # TODO: dataset kwargs?
                 dataset = pa_ds.parquet_dataset(
                     metadata_file.path,
                     filesystem=self.fs,
                 )
                 dataset_info["using_metadata_file"] = True
-                dataset_info["fragments"] = _frags = list(dataset.get_fragments())
-                dataset_info["file_sizes"] = [None for fi in _frags]
+                dataset_info["fragments"] = [
+                    FragmentWrapper(frag, self.fs) for frag in dataset.get_fragments()
+                ]
+                dataset_info["file_sizes"] = [None] * len(dataset_info["fragments"])
 
         if checksum is None:
             checksum = tokenize(all_files)
@@ -1056,15 +1066,14 @@ class ReadParquetPyarrowFS(ReadParquet):
                 filters=self.filters,
             )
             dataset_info["using_metadata_file"] = False
-            dataset_info["fragments"] = dataset.fragments
+            dataset_info["fragments"] = [
+                FragmentWrapper(frag, self.fs) for frag in dataset.fragments
+            ]
             dataset_info["all_files"] = all_files
 
-        dataset_info["dataset"] = dataset
         dataset_info["schema"] = dataset.schema
         dataset_info["base_meta"] = dataset.schema.empty_table().to_pandas()
-        self.operands[type(self)._parameters.index("_dataset_info_cache")] = (
-            dataset_info
-        )
+        self._dataset_info_cache = dataset_info
         return dataset_info
 
     @cached_property
@@ -1126,14 +1135,16 @@ class ReadParquetPyarrowFS(ReadParquet):
         ReadParquetPyarrowFS.fragments
         """
         if self.filters is not None:
-            if self._dataset_info["using_metadata_file"]:
-                ds = self._dataset_info["dataset"]
-            else:
-                ds = self._dataset_info["dataset"]._dataset
-            return np.array(
-                list(ds.get_fragments(filter=pq.filters_to_expression(self.filters)))
+            filter_expression = pq.filters_to_expression(self.filters)
+            frags = [frag.fragment for frag in self._dataset_info["fragments"]]
+            ds = pyarrow.dataset.FileSystemDataset(
+                frags,
+                self._dataset_info["schema"],
+                format=frags[0].format,
+                filesystem=self.fs,
             )
-        return np.array(self._dataset_info["fragments"])
+            return np.array(list(ds.get_fragments(filter=filter_expression)))
+        return np.array([frag.fragment for frag in self._dataset_info["fragments"]])
 
     @property
     def _fusion_compression_factor(self):
@@ -1265,6 +1276,7 @@ class ReadParquetFSSpec(ReadParquet):
         "_partitions",
         "_series",
         "_dataset_info_cache",
+        "_pq_length_stats",
     ]
     _defaults = {
         "columns": None,
@@ -1285,6 +1297,7 @@ class ReadParquetFSSpec(ReadParquet):
         "_partitions": None,
         "_series": False,
         "_dataset_info_cache": None,
+        "_pq_length_stats": None,
     }
 
     @property
@@ -1396,9 +1409,7 @@ class ReadParquetFSSpec(ReadParquet):
         dataset_info["all_columns"] = all_columns
         dataset_info["calculate_divisions"] = self.calculate_divisions
 
-        self.operands[type(self)._parameters.index("_dataset_info_cache")] = (
-            dataset_info
-        )
+        self._dataset_info_cache = dataset_info
         return dataset_info
 
     def _filtered_task(self, name: Key, index: int) -> Task:
@@ -1465,30 +1476,27 @@ class ReadParquetFSSpec(ReadParquet):
     def _get_lengths(self) -> tuple | None:
         """Return known partition lengths using parquet statistics"""
         if not self.filters:
-            self._update_length_statistics()
-            return tuple(  # type: ignore
+            return tuple(
                 length
-                for i, length in enumerate(self._pq_length_stats)  # type: ignore
+                for i, length in enumerate(self._pq_length_stats)
                 if not self._filtered or i in self._partitions
             )
         return None
 
-    def _update_length_statistics(self):
+    @cached_property
+    def _pq_length_stats(self):
         """Ensure that partition-length statistics are up to date"""
 
-        if not self._pq_length_stats:
-            if self._plan["statistics"]:
-                # Already have statistics from original API call
-                self._pq_length_stats = tuple(
-                    stat["num-rows"]
-                    for i, stat in enumerate(self._plan["statistics"])
-                    if not self._filtered or i in self._partitions
-                )
-            else:
-                # Need to go back and collect statistics
-                self._pq_length_stats = tuple(
-                    stat["num-rows"] for stat in _collect_pq_statistics(self)
-                )
+        if self._plan["statistics"]:
+            # Already have statistics from original API call
+            return tuple(
+                stat["num-rows"]
+                for i, stat in enumerate(self._plan["statistics"])
+                if not self._filtered or i in self._partitions
+            )
+        else:
+            # Need to go back and collect statistics
+            return tuple(stat["num-rows"] for stat in _collect_pq_statistics(self))
 
 
 #
@@ -1977,6 +1985,8 @@ def _aggregate_statistics_to_file(stats):
 def _gather_statistics(frags):
     @dask.delayed
     def _collect_statistics(token_fragment):
+        if isinstance(token_fragment[1], FragmentWrapper):
+            token_fragment[1] = token_fragment[1].fragment
         return token_fragment[0], _extract_stats(token_fragment[1].metadata.to_dict())
 
     return dask.compute(
