@@ -1,19 +1,13 @@
 from __future__ import annotations
 
-import contextlib
-import itertools
 import multiprocessing as mp
-import os
 import pickle
 import random
 import string
-import tempfile
 from concurrent.futures import ProcessPoolExecutor
-from copy import copy
 from datetime import date, time
 from decimal import Decimal
 from functools import partial
-from unittest import mock
 
 import numpy as np
 import pandas as pd
@@ -23,25 +17,14 @@ import dask
 import dask.dataframe as dd
 from dask.base import compute_as_if_collection
 from dask.dataframe._compat import PANDAS_GE_220, assert_categorical_equal, tm
-from dask.dataframe.shuffle import (
-    _calculate_divisions,
-    _noop,
-    maybe_buffered_partd,
-    partitioning_index,
-    rearrange_by_column,
-    rearrange_by_divisions,
-    shuffle,
-)
+from dask.dataframe.shuffle import maybe_buffered_partd, partitioning_index
 from dask.dataframe.utils import assert_eq, make_meta
-from dask.optimization import cull
 
 try:
     import pyarrow as pa
 except ImportError:
     pa = None
 
-DASK_EXPR_ENABLED = dd._dask_expr_enabled()
-
 dsk = {
     ("x", 0): pd.DataFrame({"a": [1, 2, 3], "b": [1, 4, 7]}, index=[0, 1, 3]),
     ("x", 1): pd.DataFrame({"a": [4, 5, 6], "b": [2, 5, 8]}, index=[5, 6, 8]),
@@ -50,15 +33,9 @@ dsk = {
 meta = make_meta(
     {"a": "i8", "b": "i8"}, index=pd.Index([], "i8"), parent_meta=pd.DataFrame()
 )
-if DASK_EXPR_ENABLED:
-    d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
-    shuffle_func = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
-    shuffle = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
-else:
-    d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
-    shuffle_func = (
-        shuffle  # type: ignore[assignment]  # conflicts with keyword argument
-    )
+d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
+shuffle_func = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
+shuffle = lambda df, *args, **kwargs: df.shuffle(*args, **kwargs)
 full = d.compute()
 
 
@@ -66,30 +43,11 @@ def test_shuffle(shuffle_method):
     s = shuffle_func(d, d.b, shuffle_method=shuffle_method)
     assert isinstance(s, dd.DataFrame)
     assert s.npartitions == d.npartitions
-
-    if not DASK_EXPR_ENABLED:
-        x = dask.get(s.dask, (s._name, 0))
-        y = dask.get(s.dask, (s._name, 1))
-
-        assert not (set(x.b) & set(y.b))  # disjoint
     assert set(s.dask).issuperset(d.dask)
 
     assert shuffle_func(d, d.b)._name == shuffle_func(d, d.b)._name
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="no deprecation necessary")
-def test_shuffle_deprecated_shuffle_keyword(shuffle_method):
-    from dask.dataframe.tests.test_multi import list_eq
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        result = dd.shuffle.shuffle(d, d.b, shuffle=shuffle_method)
-    list_eq(result, d)
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        result = d.shuffle(d.b, shuffle=shuffle_method)
-    list_eq(result, d)
-
-
 def test_default_partitions():
     assert shuffle(d, d.b).npartitions == d.npartitions
 
@@ -234,11 +192,6 @@ def test_set_index_general(npartitions, shuffle_method):
         df.set_index(df.x + 1), ddf.set_index(ddf.x + 1, shuffle_method=shuffle_method)
     )
 
-    if DASK_EXPR_ENABLED:
-        return
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        assert_eq(df.set_index("x"), ddf.set_index("x", shuffle=shuffle_method))
-
 
 @pytest.mark.parametrize(
     "string_dtype", ["string[python]", "string[pyarrow]", "object"]
@@ -266,13 +219,7 @@ def test_set_index_self_index(shuffle_method):
     )
 
     a = dd.from_pandas(df, npartitions=4)
-    if DASK_EXPR_ENABLED:
-        ctx = contextlib.nullcontext()
-    else:
-        ctx = pytest.warns(UserWarning, match="this is a no-op")
-
-    with ctx:
-        b = a.set_index(a.index, shuffle_method=shuffle_method)
+    b = a.set_index(a.index, shuffle_method=shuffle_method)
     assert a is b
 
     assert_eq(b, df.set_index(df.index))
@@ -370,81 +317,6 @@ def test_shuffle_sort(shuffle_method):
     assert_eq(ddf2, df2, sort_results=False)
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
-@pytest.mark.parametrize("scheduler", ["threads", "processes"])
-def test_rearrange(shuffle_method, scheduler):
-    df = pd.DataFrame({"x": np.random.random(10)})
-    ddf = dd.from_pandas(df, npartitions=4)
-    ddf2 = ddf.assign(_partitions=ddf.x % 4)
-
-    result = rearrange_by_column(
-        ddf2, "_partitions", max_branch=32, shuffle_method=shuffle_method
-    )
-    assert result.npartitions == ddf.npartitions
-    assert set(ddf.dask).issubset(result.dask)
-
-    # Every value in exactly one partition
-    a = result.compute(scheduler=scheduler)
-    get = dask.base.get_scheduler(scheduler=scheduler)
-    parts = get(result.dask, result.__dask_keys__())
-
-    for i in a._partitions.drop_duplicates():
-        assert sum(i in set(part._partitions) for part in parts) == 1
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
-def test_rearrange_cleanup():
-    df = pd.DataFrame({"x": np.random.random(10)})
-    ddf = dd.from_pandas(df, npartitions=4)
-    ddf2 = ddf.assign(_partitions=ddf.x % 4)
-
-    tmpdir = tempfile.mkdtemp()
-
-    with dask.config.set(temporay_directory=str(tmpdir)):
-        result = rearrange_by_column(
-            ddf2, "_partitions", max_branch=32, shuffle_method="disk"
-        )
-        result.compute(scheduler="processes")
-
-    assert len(os.listdir(tmpdir)) == 0
-
-
-def mock_shuffle_group_3(df, col, npartitions, p):
-    raise ValueError("Mock exception!")
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
-def test_rearrange_disk_cleanup_with_exception():
-    # ensure temporary files are cleaned up when there's an internal exception.
-
-    with mock.patch("dask.dataframe.shuffle.shuffle_group_3", new=mock_shuffle_group_3):
-        df = pd.DataFrame({"x": np.random.random(10)})
-        ddf = dd.from_pandas(df, npartitions=4)
-        ddf2 = ddf.assign(_partitions=ddf.x % 4)
-
-        tmpdir = tempfile.mkdtemp()
-
-        with dask.config.set(temporay_directory=str(tmpdir)):
-            with pytest.raises(ValueError, match="Mock exception!"):
-                result = rearrange_by_column(
-                    ddf2, "_partitions", max_branch=32, shuffle_method="disk"
-                )
-                result.compute(scheduler="processes")
-
-    assert len(os.listdir(tmpdir)) == 0
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
-def test_rearrange_by_column_with_narrow_divisions():
-    from dask.dataframe.tests.test_multi import list_eq
-
-    A = pd.DataFrame({"x": [1, 2, 3, 4, 5, 6], "y": [1, 1, 2, 2, 3, 4]})
-    a = dd.repartition(A, [0, 4, 5])
-
-    df = rearrange_by_divisions(a, "x", (0, 2, 5))
-    list_eq(df, a)
-
-
 def test_maybe_buffered_partd(tmp_path):
     import partd
 
@@ -508,65 +380,6 @@ def test_set_index_divisions_2():
     assert list(result.compute(scheduler="sync").index[-2:]) == ["d", "d"]
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not available")
-def test_set_index_divisions_compute():
-    deprecated = pytest.warns(
-        FutureWarning, match="the 'compute' keyword is deprecated"
-    )
-
-    with deprecated:
-        d2 = d.set_index("b", divisions=[0, 2, 9], compute=False)
-    with deprecated:
-        d3 = d.set_index("b", divisions=[0, 2, 9], compute=True)
-
-    assert_eq(d2, d3)
-    assert_eq(d2, full.set_index("b"))
-    assert_eq(d3, full.set_index("b"))
-    assert len(d2.dask) > len(d3.dask)
-
-    with deprecated:
-        d4 = d.set_index(d.b, divisions=[0, 2, 9], compute=False)
-    with deprecated:
-        d5 = d.set_index(d.b, divisions=[0, 2, 9], compute=True)
-    exp = full.copy()
-    exp.index = exp.b
-    assert_eq(d4, d5)
-    assert_eq(d4, exp)
-    assert_eq(d5, exp)
-    assert len(d4.dask) > len(d5.dask)
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="constructor doesn't work")
-def test_set_index_divisions_sorted():
-    p1 = pd.DataFrame({"x": [10, 11, 12], "y": ["a", "a", "a"]})
-    p2 = pd.DataFrame({"x": [13, 14, 15], "y": ["b", "b", "c"]})
-    p3 = pd.DataFrame({"x": [16, 17, 18], "y": ["d", "e", "e"]})
-
-    ddf = dd.DataFrame(
-        {("x", 0): p1, ("x", 1): p2, ("x", 2): p3}, "x", p1, [None, None, None, None]
-    )
-    df = ddf.compute()
-
-    def throw(*args, **kwargs):
-        raise Exception("Shouldn't have computed")
-
-    with dask.config.set(scheduler=throw):
-        res = ddf.set_index("x", divisions=[10, 13, 16, 18], sorted=True)
-    assert_eq(res, df.set_index("x"))
-
-    with dask.config.set(scheduler=throw):
-        res = ddf.set_index("y", divisions=["a", "b", "d", "e"], sorted=True)
-    assert_eq(res, df.set_index("y"))
-
-    # with sorted=True, divisions must be same length as df.divisions
-    with pytest.raises(ValueError):
-        ddf.set_index("y", divisions=["a", "b", "c", "d", "e"], sorted=True)
-
-    # Divisions must be sorted
-    with pytest.raises(ValueError):
-        ddf.set_index("y", divisions=["a", "b", "d", "c"], sorted=True)
-
-
 @pytest.mark.slow
 def test_set_index_consistent_divisions():
     # See https://github.com/dask/dask/issues/3867
@@ -592,38 +405,9 @@ def make_part(n):
     return pd.DataFrame({"x": np.random.random(n), "y": np.random.random(n)})
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="auto not supported")
-def test_npartitions_auto_raises_deprecation_warning():
-    df = pd.DataFrame({"x": range(100), "y": range(100)})
-    ddf = dd.from_pandas(df, npartitions=10, name="x", sort=False)
-    with pytest.warns(FutureWarning, match="npartitions='auto'"):
-        ddf.set_index("x", npartitions="auto")
-    with pytest.warns(FutureWarning, match="npartitions='auto'"):
-        ddf.sort_values(by=["x"], npartitions="auto")
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="auto not supported")
-def test_set_index_doesnt_increase_partitions(shuffle_method):
-    nparts = 2
-    nbytes = 1e6
-    n = int(nbytes / (nparts * 8))
-    ddf = dd.DataFrame(
-        {("x", i): (make_part, n) for i in range(nparts)},
-        "x",
-        make_part(1),
-        [None] * (nparts + 1),
-    )
-    ddf2 = ddf.set_index("x", shuffle_method=shuffle_method, partition_size=nbytes)
-    assert ddf2.npartitions <= ddf.npartitions
-
-
 def test_set_index_detects_sorted_data(shuffle_method):
     df = pd.DataFrame({"x": range(100), "y": range(100)})
-    if DASK_EXPR_ENABLED:
-        ddf = dd.from_pandas(df, npartitions=10, sort=False)
-    else:
-        ddf = dd.from_pandas(df, npartitions=10, name="x", sort=False)
-
+    ddf = dd.from_pandas(df, npartitions=10, sort=False)
     ddf2 = ddf.set_index("x", shuffle_method=shuffle_method)
     assert len(ddf2.dask) < ddf.npartitions * 4
 
@@ -722,11 +506,7 @@ def test_set_index(engine):
         ("x", 2): pd.DataFrame({"a": [7, 8, 9], "b": [9, 1, 8]}, index=[9, 9, 9]),
     }
 
-    if DASK_EXPR_ENABLED:
-        d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
-    else:
-        d = dd.DataFrame(dsk, "x", meta, [0, 4, 9, 9])
-
+    d = dd.repartition(pd.concat(dsk.values()), [0, 4, 9, 9])
     full = d.compute()
 
     d2 = d.set_index("b", npartitions=3)
@@ -748,22 +528,6 @@ def test_set_index(engine):
     assert_eq(d5, full.set_index(["b"]))
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not deprecated")
-def test_set_index_deprecated_shuffle_keyword(shuffle_method):
-    df = pd.DataFrame({"x": [4, 1, 1, 3, 3], "y": [1.0, 1, 1, 1, 2]})
-    ddf = dd.from_pandas(df, 2)
-
-    expected = df.set_index("x")
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        result = ddf.set_index("x", shuffle=shuffle_method)
-    assert_eq(result, expected)
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        result = dd.shuffle.set_index(ddf, "x", shuffle=shuffle_method)
-    assert_eq(result, expected)
-
-
 @pytest.mark.parametrize(
     "engine", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
 )
@@ -817,62 +581,6 @@ def test_set_index_interpolate_int(engine):
     assert all(np.issubdtype(type(x), np.integer) for x in d1.divisions)
 
 
-@pytest.mark.skipif(
-    DASK_EXPR_ENABLED, reason="we don't do division inference for 1 partition frames"
-)
-@pytest.mark.parametrize(
-    "engine", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
-)
-def test_set_index_interpolate_large_uint(engine):
-    if engine == "cudf":
-        # NOTE: engine == "cudf" requires cudf/dask_cudf,
-        # will be skipped by non-GPU CI.
-
-        cudf = pytest.importorskip("cudf")
-        dask_cudf = pytest.importorskip("dask_cudf")
-
-    """This test is for #7304"""
-    df = pd.DataFrame(
-        {"x": np.array([612509347682975743, 616762138058293247], dtype=np.uint64)}
-    )
-
-    if engine == "cudf":
-        gdf = cudf.from_pandas(df)
-        d = dask_cudf.from_cudf(gdf, npartitions=1)
-    else:
-        d = dd.from_pandas(df, 1)
-
-    d1 = d.set_index("x", npartitions=1)
-    assert d1.npartitions == 1
-    assert set(d1.divisions) == {612509347682975743, 616762138058293247}
-
-
-@pytest.mark.skipif(
-    DASK_EXPR_ENABLED, reason="we don't do division inference for 1 partition frames"
-)
-def test_set_index_timezone():
-    s_naive = pd.Series(pd.date_range("20130101", periods=3))
-    s_aware = pd.Series(pd.date_range("20130101", periods=3, tz="US/Eastern"))
-    df = pd.DataFrame({"tz": s_aware, "notz": s_naive})
-    d = dd.from_pandas(df, npartitions=1)  # TODO: Use npartitions=2
-
-    d1 = d.set_index("notz", npartitions=1)
-    s1 = pd.DatetimeIndex(s_naive.values, dtype=s_naive.dtype)
-    assert d1.divisions[0] == s_naive[0] == s1[0]
-    assert d1.divisions[-1] == s_naive[2] == s1[2]
-
-    # We currently lose "freq".  Converting data with pandas-defined dtypes
-    # to numpy or pure Python can be lossy like this.
-    d2 = d.set_index("tz", npartitions=1)
-    s2 = pd.DatetimeIndex(s_aware, dtype=s_aware.dtype)
-    assert d2.divisions[0] == s2[0]
-    assert d2.divisions[-1] == s2[2]
-    assert d2.divisions[0].tz == s2[0].tz
-    assert d2.divisions[0].tz is not None
-    s2badtype = pd.DatetimeIndex(s_aware.values, dtype=s_naive.dtype)
-    assert not d2.divisions[0] == s2badtype[0]
-
-
 def test_set_index_npartitions():
     # https://github.com/dask/dask/issues/6974
     data = pd.DataFrame(
@@ -973,11 +681,6 @@ def test_set_index_sorted_true():
             df.set_index(df.x + 1, drop=drop),
         )
 
-    if not DASK_EXPR_ENABLED:
-        # we don't validate this
-        with pytest.raises(ValueError):
-            a.set_index(a.z, sorted=True)
-
 
 def test_set_index_sorted_single_partition():
     df = pd.DataFrame({"x": [1, 2, 3, 4], "y": [1, 0, 1, 0]})
@@ -1036,13 +739,6 @@ def test_set_index_on_empty(converter):
     assert actual.npartitions == 1
     assert all(pd.isnull(d) for d in actual.divisions)
 
-    if not DASK_EXPR_ENABLED:
-        # we don't recompute divisions in dask-expr
-        actual = ddf[ddf.y > df.y.max()].set_index("x", sorted=True)
-        assert assert_eq(actual, expected, check_freq=False)
-        assert actual.npartitions == 1
-        assert all(pd.isnull(d) for d in actual.divisions)
-
 
 def test_set_index_categorical():
     # https://github.com/dask/dask/issues/5671
@@ -1078,8 +774,6 @@ def test_set_index_with_empty_and_overlap():
 
 
 def test_compute_divisions():
-    from dask.dataframe.shuffle import compute_and_set_divisions
-
     df = pd.DataFrame(
         {"x": [1, 2, 3, 4], "y": [10, 20, 20, 40], "z": [4, 3, 2, 1]},
         index=[1, 3, 10, 20],
@@ -1087,11 +781,7 @@ def test_compute_divisions():
     a = dd.from_pandas(df, 2, sort=False).clear_divisions()
     assert not a.known_divisions
 
-    if DASK_EXPR_ENABLED:
-        b = a.compute_current_divisions(set_divisions=True)
-    else:
-        b = compute_and_set_divisions(copy(a))
-
+    b = a.compute_current_divisions(set_divisions=True)
     assert_eq(a, b, check_divisions=False)
     assert b.known_divisions
 
@@ -1129,41 +819,6 @@ def test_gh_2730():
     tm.assert_frame_equal(result.sort_values("KEY").reset_index(drop=True), expected)
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="we test this over in dask-expr")
-def test_set_index_does_not_repeat_work_due_to_optimizations():
-    # Atomic counter
-    count = itertools.count()
-
-    def increment():
-        next(count)
-
-    def make_part(dummy, n):
-        return pd.DataFrame({"x": np.random.random(n), "y": np.random.random(n)})
-
-    nbytes = 1e6
-    nparts = 50
-    n = int(nbytes / (nparts * 8))
-
-    dsk = {("inc", i): (increment,) for i in range(nparts)}
-    dsk.update({("x", i): (make_part, ("inc", i), n) for i in range(nparts)})
-    ddf = dd.DataFrame(dsk, "x", make_part(None, 1), [None] * (nparts + 1))
-
-    ddf.set_index("x")
-    ntimes = next(count)
-    assert ntimes == nparts
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="we don't support inplace")
-def test_set_index_errors_with_inplace_kwarg():
-    df = pd.DataFrame({"a": [9, 8, 7], "b": [6, 5, 4], "c": [3, 2, 1]})
-    ddf = dd.from_pandas(df, npartitions=1)
-
-    ddf.set_index("a")
-
-    with pytest.raises(NotImplementedError):
-        ddf.set_index("a", inplace=True)
-
-
 def test_set_index_timestamp():
     df = pd.DataFrame({"A": pd.date_range("2000", periods=12, tz="US/Central"), "B": 1})
     ddf = dd.from_pandas(df, 2)
@@ -1316,23 +971,12 @@ def test_compute_current_divisions_nan_partition():
     a = d[d.a > 3].sort_values("a")
     divisions = a.compute_current_divisions("a")
     assert divisions == (4, 5, 8, 9)
-
-    if DASK_EXPR_ENABLED:
-        # We don't support this
-        pass
-    else:
-        a.divisions = divisions
     assert_eq(a, a, check_divisions=False)
 
     # Compute divisions with 0 null partitions
     a = d[d.a > 1].sort_values("a")
     divisions = a.compute_current_divisions("a")
     assert divisions == (2, 4, 7, 9)
-    if DASK_EXPR_ENABLED:
-        # We don't support this
-        pass
-    else:
-        a.divisions = divisions
     assert_eq(a, a, check_divisions=False)
 
 
@@ -1359,91 +1003,6 @@ def test_compute_current_divisions_overlap_2():
         ddf2.compute_current_divisions()
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
-def test_shuffle_hlg_layer():
-    # This test checks that the `ShuffleLayer` HLG Layer
-    # is used (as expected) for a multi-stage shuffle.
-    ddf = dd.from_pandas(
-        pd.DataFrame({"a": np.random.randint(0, 10, 100)}), npartitions=10
-    )
-    # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle_method="tasks")
-    keys = [(ddf_shuffled._name, i) for i in range(ddf_shuffled.npartitions)]
-
-    # Cull the HLG
-    dsk = ddf_shuffled.__dask_graph__()
-    dsk_culled = dsk.cull(set(keys))
-    assert isinstance(dsk_culled, dask.highlevelgraph.HighLevelGraph)
-
-    # Ensure we have ShuffleLayers
-    assert any(
-        isinstance(layer, dd.shuffle.ShuffleLayer) for layer in dsk.layers.values()
-    )
-
-    # Check that the ShuffleLayers are non-materialized
-    for layer in dsk.layers.values():
-        if isinstance(layer, dd.shuffle.ShuffleLayer):
-            assert not hasattr(layer, "_cached_dict")
-
-    # Make sure HLG culling reduces the graph size
-    assert len(dsk_culled) < len(dsk)
-
-    # Check ShuffleLayer names
-    for name, layer in dsk.layers.items():
-        if isinstance(layer, dd.shuffle.ShuffleLayer):
-            assert name.startswith("shuffle-")
-
-    # Since we already culled the HLG,
-    # culling the dictionary should not change the graph
-    dsk_dict = dict(dsk_culled)
-    dsk_dict_culled, _ = cull(dsk_dict, keys)
-    assert dsk_dict_culled == dsk_dict
-
-
-def test_shuffle_partitions_meta_dtype():
-    ddf = dd.from_pandas(
-        pd.DataFrame({"a": np.random.randint(0, 10, 100)}, index=np.random.random(100)),
-        npartitions=10,
-    )
-    # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle(ddf["a"] % 10, max_branch=3, shuffle_method="tasks")
-    # Cull the HLG
-    if not DASK_EXPR_ENABLED:
-        dsk = ddf_shuffled.__dask_graph__()
-
-        for layer in dsk.layers.values():
-            if isinstance(layer, dd.shuffle.ShuffleLayer):
-                assert layer.meta_input["_partitions"].dtype == np.int64
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
-@pytest.mark.parametrize(
-    "npartitions",
-    [
-        10,  # ShuffleLayer
-        1,  # SimpleShuffleLayer
-    ],
-)
-def test_shuffle_hlg_layer_serialize(npartitions):
-    ddf = dd.from_pandas(
-        pd.DataFrame({"a": np.random.randint(0, 10, 100)}), npartitions=npartitions
-    )
-    # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf_shuffled = ddf.shuffle("a", max_branch=3, shuffle_method="tasks")
-
-    # Ensure shuffle layers can be serialized and don't result in
-    # the underlying low-level graph being materialized
-    dsk = ddf_shuffled.__dask_graph__()
-    for layer in dsk.layers.values():
-        if not isinstance(layer, dd.shuffle.SimpleShuffleLayer):
-            continue
-        assert not hasattr(layer, "_cached_dict")
-        layer_roundtrip = pickle.loads(pickle.dumps(layer))
-        assert type(layer_roundtrip) == type(layer)
-        assert not hasattr(layer_roundtrip, "_cached_dict")
-        assert layer_roundtrip.keys() == layer.keys()
-
-
 def test_set_index_nan_partition():
     d[d.a > 3].set_index("a")  # Set index with 1 null partition
     d[d.a > 1].set_index("a", sorted=True)  # Set sorted index with 0 null partitions
@@ -1508,45 +1067,6 @@ def test_set_index_with_series_uses_fastpath():
     assert_eq(res, expected)
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
-def test_set_index_partitions_meta_dtype():
-    ddf = dd.from_pandas(
-        pd.DataFrame({"a": np.random.randint(0, 10, 100)}, index=np.random.random(100)),
-        npartitions=10,
-    )
-    # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-    ddf = ddf.set_index("a", shuffle_method="tasks")
-    # Cull the HLG
-    dsk = ddf.__dask_graph__()
-
-    for layer in dsk.layers.values():
-        if isinstance(layer, dd.shuffle.SimpleShuffleLayer):
-            assert layer.meta_input["_partitions"].dtype == np.int64
-
-
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not valid for dask-expr")
-def test_sort_values_partitions_meta_dtype_with_divisions():
-    with dask.config.set({"dataframe.shuffle.method": "tasks"}):
-        ddf = dd.from_pandas(
-            pd.DataFrame(
-                {
-                    "a": np.random.randint(0, 10, 100),
-                    "b": np.random.randint(0, 10, 100),
-                },
-                index=np.random.random(100),
-            ),
-            npartitions=10,
-        )
-        # Disk-based shuffle doesn't use HLG layers at the moment, so we only test tasks
-        ddf = ddf.set_index("a", shuffle_method="tasks").sort_values("b")
-        # Cull the HLG
-        dsk = ddf.__dask_graph__()
-
-        for layer in dsk.layers.values():
-            if isinstance(layer, dd.shuffle.SimpleShuffleLayer):
-                assert layer.meta_input["_partitions"].dtype == np.int64
-
-
 @pytest.mark.parametrize("ascending", [True, False])
 @pytest.mark.parametrize("by", ["a", "b", ["a", "b"]])
 @pytest.mark.parametrize("nelem", [10, 500])
@@ -1564,20 +1084,6 @@ def test_sort_values(nelem, by, ascending):
     dd.assert_eq(got, expect, check_index=False, sort_results=False)
 
 
-@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="not deprecated")
-def test_sort_values_deprecated_shuffle_keyword(shuffle_method):
-    np.random.seed(0)
-    df = pd.DataFrame()
-    df["a"] = np.ascontiguousarray(np.arange(10)[::-1])
-    df["b"] = np.arange(100, 10 + 100)
-    ddf = dd.from_pandas(df, npartitions=10)
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        got = ddf.sort_values(by=["a"], shuffle=shuffle_method)
-    expect = df.sort_values(by=["a"])
-    dd.assert_eq(got, expect, check_index=False, sort_results=False)
-
-
 @pytest.mark.parametrize(
     "backend", ["pandas", pytest.param("cudf", marks=pytest.mark.gpu)]
 )
@@ -1599,16 +1105,6 @@ def test_sort_values_tasks_backend(backend, by, ascending):
     )
     dd.assert_eq(got, expect, sort_results=False)
 
-    if DASK_EXPR_ENABLED:
-        return
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        got = dd.DataFrame.sort_values(ddf, by=by, ascending=ascending, shuffle="tasks")
-    dd.assert_eq(got, expect, sort_results=False)
-
-    with pytest.warns(FutureWarning, match="'shuffle' keyword is deprecated"):
-        got = ddf.sort_values(by=by, ascending=ascending, shuffle="tasks")
-    dd.assert_eq(got, expect, sort_results=False)
-
 
 @pytest.mark.parametrize("ascending", [True, False, [False, True], [True, False]])
 @pytest.mark.parametrize("by", [["a", "b"], ["b", "a"]])
@@ -1673,11 +1169,6 @@ def test_shuffle_by_as_list():
     dd.assert_eq(got, expect, check_index=False)
 
 
-def test_noop():
-    assert _noop(1, None) == 1
-    assert _noop("test", None) == "test"
-
-
 @pytest.mark.parametrize("by", [["a", "b"], ["b", "a"]])
 @pytest.mark.parametrize("nparts", [1, 10])
 def test_sort_values_custom_function(by, nparts):
@@ -1742,42 +1233,6 @@ def test_sort_values_timestamp(npartitions):
     assert_eq(result, expected)
 
 
-@pytest.mark.parametrize(
-    "pdf,expected",
-    [
-        (
-            pd.DataFrame({"x": list("aabbcc"), "y": list("xyyyzz")}),
-            (["a", "b", "c", "c"], ["a", "b", "c", "c"], ["a", "b", "c", "c"], False),
-        ),
-        (
-            pd.DataFrame(
-                {
-                    "x": [1, 0, 1, 3, 4, 5, 7, 8, 1, 2, 3],
-                    "y": [21, 9, 7, 8, 3, 5, 4, 5, 6, 3, 10],
-                }
-            ),
-            ([0, 1, 2, 4, 8], [0, 3, 1, 2], [1, 5, 8, 3], False),
-        ),
-        (
-            pd.DataFrame({"x": [5, 6, 7, 10, None, 10, 2, None, 8, 4, None]}),
-            (
-                [2.0, 4.0, 5.666666666666667, 8.0, 10.0],
-                [5.0, 10.0, 2.0, 4.0],
-                [7.0, 10.0, 8.0, 4.0],
-                False,
-            ),
-        ),
-    ],
-)
-def test_calculate_divisions(pdf, expected):
-    ddf = dd.from_pandas(pdf, npartitions=4)
-    divisions, mins, maxes, presorted = _calculate_divisions(ddf, ddf["x"], False, 4)
-    assert divisions == expected[0]
-    assert mins == expected[1]
-    assert maxes == expected[2]
-    assert presorted == expected[3]
-
-
 @pytest.mark.skipif(pa is None, reason="Need pyarrow")
 @pytest.mark.parametrize(
     "data, dtype",
